/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.model;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.hibernate.annotations.BatchSize;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

import javax.persistence.*;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
@Entity
@Table(name = "projects")
@NamedQueries(
    {
    @NamedQuery(name = MavenProject.SQL_SELECT_BY_ID, query = "SELECT p FROM MavenProject p WHERE p.id = :id"),
    @NamedQuery(name = MavenProject.SQL_SELECT_DISABLED_PROJECTS, query = "SELECT p FROM MavenProject p WHERE p.enabled=false AND " +
    		"SCOPE='" + MavenProject.SCOPE_PROJECT + "' AND " +
    		"p.rootId IS NULL")
}
)
public class MavenProject implements Serializable {
  
  public static final String SCOPE_PROJECT = "PRJ";
  public static final String SCOPE_PACKAGE = "PAC";
  public static final String SCOPE_CLASS = "CLA";

  public final static String SQL_SELECT_BY_ID = "MavenProject.selectById";
  public static final String SQL_SELECT_DISABLED_PROJECTS = "MavenProject.selectDisabledProjects";

  private static final long serialVersionUID = -7747333646654989328L;
  public static final int DESCRIPTION_COLUMN_SIZE = 2000;
  public static final int NAME_COLUMN_SIZE = 256;
  public static final int KEY_SIZE = 230;

  @Id
  @Column(name = "id")
  @SequenceGenerator(name = "PROJECTS_SEQ", sequenceName = "PROJECTS_SEQ")
  @GeneratedValue(strategy = GenerationType.AUTO, generator = "PROJECTS_SEQ")
  private Integer id;

  @Column(name = "name", updatable = true, nullable = true, length = NAME_COLUMN_SIZE)
  private String name;

  @Column(name = "description", updatable = true, nullable = true, length = DESCRIPTION_COLUMN_SIZE)
  private String description;

  @Column(name = "enabled", updatable = true, nullable = false)
  private Boolean enabled = Boolean.TRUE;

  @Column(name = "scope", updatable = false, nullable = false, length = 3)
  private String scope;

  @Column(name = "qualifier", updatable = false, nullable = true, length = 3)
  private String qualifier;


  @Column(name = "kee", updatable = false, nullable = false, length = KEY_SIZE)
  private String key;

  @Column(name = "root_id", updatable = true, nullable = true)
  private Integer rootId;

  @OneToMany(mappedBy = "mavenProject", fetch = FetchType.LAZY, cascade = {CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REMOVE})
  @BatchSize(size = 8)
  private List<ProjectLink> projectLinks = new ArrayList<ProjectLink>();


  public MavenProject() {
  }

  public MavenProject(String scope, String key, String qualifier, Integer rootId, String name) {
    this.scope = scope;
    this.key = key;
    this.rootId = rootId;
    this.name = name;
    this.qualifier = qualifier;
  }

  public static MavenProject newMavenProject(String groupId, String artifactId, String branch, String name) {
    MavenProject project = new MavenProject();
    project.setScope(SCOPE_PROJECT);
    project.setMavenId(groupId, artifactId, branch);
    project.setName(name);
    return project;
  }

  public static String toMavenKey(String groupId, String artifactId, String branch) {
    StringBuilder sb = new StringBuilder().append(groupId).append(":").append(artifactId);
    if (branch != null) {
      sb.append(":").append(branch);
    }
    return sb.toString();
  }

  public String[] getMavenId() {
    if (SCOPE_PROJECT.equals(scope)) {
      return StringUtils.split(key, ":");
    }
    return null;
  }

  public void setMavenId(String groupId, String artifactId, String branch) {
    key = toMavenKey(groupId, artifactId, branch);
    if (key.length() > KEY_SIZE) {
      throw new IllegalArgumentException("key is too long, max is " + KEY_SIZE + " characters : " + key);
    }

  }

  public String getMavenGroupId() {
    String[] mavenId = getMavenId();
    if (mavenId != null) {
      return mavenId[0];
    }
    return null;
  }

  public String getMavenArtifactId() {
    String[] mavenId = getMavenId();
    if (mavenId != null) {
      return mavenId[1];
    }
    return null;
  }

  public String getMavenBranch() {
    String[] mavenId = getMavenId();
    if (mavenId != null && mavenId.length > 2) {
      return mavenId[2];
    }
    return null;
  }

  public List<ProjectLink> getProjectLinks() {
    return projectLinks;
  }

  public void setProjectLinks(List<ProjectLink> projectLinks) {
    this.projectLinks = projectLinks;
  }

  public String getDescription() {
    return description;
  }

  public void setDescription(String description) {
    this.description = StringUtils.abbreviate(description, DESCRIPTION_COLUMN_SIZE);
  }

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = StringUtils.abbreviate(name, NAME_COLUMN_SIZE);
  }

  public Integer getId() {
    return id;
  }

  public void setId(Integer id) {
    this.id = id;
  }

  public Boolean getEnabled() {
    return enabled;
  }

  public void setEnabled(Boolean enabled) {
    this.enabled = enabled;
  }

  public String getScope() {
    return scope;
  }

  public void setScope(String scope) {
    this.scope = scope;
  }

  public String getKey() {
    return key;
  }

  public void setKey(String key) {
    this.key = key;
  }

  public Integer getRootId() {
    return rootId;
  }

  public void setRootId(Integer rootId) {
    this.rootId = rootId;
  }

  public String getQualifier() {
    return qualifier;
  }

  public void setQualifier(String qualifier) {
    this.qualifier = qualifier;
  }

  public ProjectLink getProjectLinkByType(String type) {
    if (type == null) {
      return null;
    }
    for (ProjectLink projectLink : projectLinks) {
      if (type.equals(projectLink.getType())) {
        return projectLink;
      }
    }
    return null;
  }

  public boolean equals(Object obj) {
    if (!(obj instanceof MavenProject)) {
      return false;
    }
    if (this == obj) {
      return true;
    }
    MavenProject other = (MavenProject) obj;
    return new EqualsBuilder()
        .append(scope, other.scope)
        .append(key, other.key)
        .append(enabled, other.enabled)
        .append(rootId, other.rootId)
        .append(qualifier, other.qualifier)
        .isEquals();
  }

  public int hashCode() {
    return new HashCodeBuilder(17, 37)
        .append(scope)
        .append(key)
        .append(enabled)
        .append(qualifier)
        .append(rootId)
        .toHashCode();
  }

  public String toString() {
    return new ToStringBuilder(this)
        .append("id", id)
        .append("scope", scope)
        .append("key", key)
        .append("qualifier", qualifier)
        .append("name", name)
        .append("enabled", enabled)
        .append("rootId", rootId)
        .toString();
  }
}