/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.commons.commands;

import java.io.*;
import java.util.*;

public class CommandTask implements Runnable {

    private ProcessBuilder processbuilder = null;
    private Process process = null;

    private String directory = null;

    private Timer timer = new Timer(true);
    private TimeOutTask timeOutTask = null;
    private long timeout = 0;

    private CommandExceptionNotifier notifier = null;
    private int exitCode = -1;

    private boolean enableCollectOutput = false;
    private StringBuilder output = new StringBuilder();
    private StringBuilder errorOutput = new StringBuilder();

    private boolean exceptionThrowed = false;

    // ------------------------------------------------------------------------
    // -- Constructors
    // ------------------------------------------------------------------------
    public CommandTask() {
    }

    public CommandTask(String... args) {
        processbuilder = new ProcessBuilder(args);
    }

    // ------------------------------------------------------------------------
    // -- Public methods
    // ------------------------------------------------------------------------
    public void run() {
        try {
            if(directory != null && directory.length() > 0) {
                processbuilder.directory(new File(directory));
            }
            process = processbuilder.start();
            collectOutput();
            collectErrorOutput();

            if(timeout > 0) {
                timeOutTask = new TimeOutTask(Thread.currentThread());
                timer.schedule(timeOutTask, timeout);
            }

            exitCode = process.waitFor();

            if(timeout > 0) {
                timer.cancel();
            }

            displayCollectAllOutput();
        } catch (InterruptedException e) {
            handleException(e);
            process.destroy();
        } catch (Exception e) {
            handleException(e);
        }
    }


    public boolean timeoutExceptionCatched() {
        return exceptionThrowed && timeOutTask != null && timeOutTask.isTimeoutThrowed();
    }

    public boolean otherExceptionCatched() {
        return exceptionThrowed;
    }

    // ------------------------------------------------------------------------
    // -- Private methods
    // ------------------------------------------------------------------------
    private void displayCollectAllOutput() {
        try {
            if(notifier != null && enableCollectOutput) {
                notifier.notifyErrorOutput(getErrorOutput());
                notifier.notifyOutput(getOutput());
            }
        } catch (IOException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }

    private void handleException(Exception e) {
        exceptionThrowed = true;
        if (notifier != null) {
            notifier.notifyException(e);
        } else {
            e.printStackTrace();
        }
    }

    private void collectErrorOutput() {
        if (enableCollectOutput) {
            Runnable runnable = new Runnable() {
                public void run() {
                    try {
                        collectOutput(process.getErrorStream(),
                                errorOutput);
                    }
                    catch (IOException e) {
                        errorOutput.append(e.getMessage());
                    }
                }
            };
            new Thread(runnable).start();
        }
    }

    private void collectOutput() {
        if (enableCollectOutput) {
            Runnable runnable = new Runnable() {
                public void run() {
                    try {
                        collectOutput(process.getInputStream(), output);
                    }
                    catch (IOException e) {
                        output.append(e.getMessage());
                    }
                }
            };
            new Thread(runnable).start();
        }
    }

    private void collectOutput(InputStream inputStream,
                               StringBuilder collector) throws IOException {
        BufferedReader reader = null;
        try {
            reader =
                    new BufferedReader(new InputStreamReader(inputStream));
            String line;
            while ((line = reader.readLine()) != null) {
                collector.append(line);
                //System.out.println(line);
            }
        }
        finally {
            reader.close();
        }
    }


    // ------------------------------------------------------------------------
    // -- Getters and setters
    // ------------------------------------------------------------------------
    public long getTimeout() {
        return timeout;
    }

    public void setTimeout(long timeout) {
        this.timeout = timeout;
    }

    public CommandExceptionNotifier getNotifier() {
        return notifier;
    }

    public void setNotifier(CommandExceptionNotifier notifier) {
        this.notifier = notifier;
    }

    public TimeOutTask getTimeOutTask() {
        return timeOutTask;
    }

    public void setTimeOutTask(TimeOutTask timeOutTask) {
        this.timeOutTask = timeOutTask;
    }

    public String getOutput() throws IOException {
        return output.toString();
    }

    public String getErrorOutput() throws IOException {
        return errorOutput.toString();
    }

    public String getDirectory() {
        return directory;
    }

    public void setDirectory(String directory) {
        this.directory = directory;
    }

    public boolean isEnableCollectOutput() {
        return enableCollectOutput;
    }

    public void setEnableCollectOutput(boolean enableCollectOutput) {
        this.enableCollectOutput = enableCollectOutput;
    }

}